import abc
from typing import Generic, Type, TypeVar

from rainbowadn.hashing.hashmentionable import HashMentionable
from rainbowadn.hashing.rainbow_factory import RainbowFactory

__all__ = ('StaticMentionable', 'StaticFactory',)

StaticMentioned = TypeVar('StaticMentioned')


class StaticMentionable(HashMentionable, abc.ABC):
    @classmethod
    def from_bytes(cls: Type[StaticMentioned], source: bytes) -> StaticMentioned:
        raise NotImplementedError

    def __factory__(self) -> RainbowFactory[StaticMentioned]:
        return self.factory()

    @classmethod
    def factory(cls: Type[StaticMentioned]) -> RainbowFactory[StaticMentioned]:
        return StaticFactory(cls)


class StaticFactory(RainbowFactory[StaticMentioned], Generic[StaticMentioned]):
    def __init__(self, cls: Type[StaticMentioned]):
        self.cls = cls

    def from_bytes(self, source: bytes) -> StaticMentioned:
        assert isinstance(source, bytes)
        if issubclass(self.cls, StaticMentionable):
            return self.cls.from_bytes(source)
        else:
            raise TypeError
